*-- File: ENTRY.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give some optional entryfields
*      which can be used either 'as is' or you can even subclass them.
*
*-- Programmers: 
*
*      Ken Mayer
*      Thomas H. Arnold
*      Romain Strieff (CIS: 71333,2147)
*
*-- History: 
*
*      11/28/1996 -- First version of this custom class file.
*
*-- Instantiation: 
*
*      set procedure to entry.cc additive
*      * create/modify a form, place an instance of one of
*      * these entryfields on the form as a normal entryfield ...
*
*-- Classes:
*      More details on each of these is contained in the commentary for
*      each class ...
*
*--    AutoIncrement
*        Need to create an auto-incrementing field? This is a nice,
*        elegant method of handling it. Every time you add a new record,
*        this auto-increment field will do its job.
*
*--    BigCursorEntryField
*        When this field gets focus, the cursor changes shape
*        to a larger one.
*
*--    DateEpochEntry
*        A date entryfield that allows the user to enter dates
*        as if using the Epoch() function in DateEx.cc
*
*--    KeyFieldEntry
*        Forces entries to be unique.
*
*--    MultipleEntry
*        This is a multiple entry entryfield, sort of a replacement 
*        for the dBASE/DOS @M picture/function.
*
*--    SeqValue
*         This is a variation of AutoIncrement that ensures sequential
*         values.
*
*-- NOTES:
*
*      For custom classes, a bug exists in the form designer that
*      ignores streaming out custom properties. For these classes
*      to work, you will want to set all custom properties in one
*      of several possible locations. The first is recommended,
*      but you can also use the others, however you only need to do
*      one ...:
*
*      1) An over-ridden OPEN or READMODAL event for the form (make 
*         sure you call the original event (i.e., SUPER::OPEN() )
*      2) The form's OnOpen event
*      3) The object's OnOpen event
*----------------------------------------------------------------------

CLASS AutoIncrement(F,N) OF ENTRYFIELD(F,N) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      Place this on a form, and make sure you datalink it to a 
   *      field in one of your tables -- the field must be numeric or 
   *      character. It should handle any number of fields of forms.
   *
   *-- WARNINGS:
   *
   *   1) This (and SEQVALUE below) uses the SQL command to create
   *      the table used -- if you need to deploy an application
   *      using this class, you need to deploy the SQL portion of
   *      the BDE, or your application will fail. You may, instead
   *      modify the code to use the standard dBASE "CREATE FROM"
   *      command, but this will take a bit more work ...
   *   2) This autoincrementing class works, but is not concerned
   *      with sequential numbers (such as in a check book), which
   *      means that if a user adds a new record, and then cancels,
   *      the new number generated here is lost. If you need to
   *      have sequential numbers, use SEQVALUE (included here)
   *      instead.
   *   3) Your form must use the FORM.beginAppend() and 
   *      form.SaveRecord() events for this to work.
   *
   *-- Programmer: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual. 
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"). You may also wish to modify the SpeedTip, but to do
   *      so, you will need to do this in the code below, as it is
   *      a "protected" property, and once you have an instance of
   *      the object, you can't change it ...
   *----------------------------------------------------------------------
       

       Protect Onopen,When,Selectall,Speedtip
       this.Height    =  1.12   && Assumes Ms San Serif, 8 Point
       this.Width     = 16      && Same
       this.Function  = "Z"     && Does Not Display Zero Values
       this.Selectall = .F.
       this.Speedtip  = "Auto-Increment -- You Can't Edit Me!"
       this.When      = {;Return .F.}
       this.Onopen    = Class::AutoIncOnopen
       
       PROCEDURE IncrementIt
          *-- this control MUST have a datalink
          if empty(this.datalink)
             ?? chr(7)
             msgbox("There is no datalink for this control!",;
                    "ERROR",16)
             RETURN .F.
          endif && Empty(This.Datalink)
       
          *-- check if it already has a value
          if type("this.value")="N" .and. this.value > 0
             RETURN .F.  && Don't do anything!
          endif && Type("this.value")="N" .And. This.Value 
          if type("this.value")="C" .and. .not. empty(this.value)
             RETURN .F.  && Don't do anything!
          endif && Type("this.value")="C" .And. .Not. Empty
        
          *-- if table doesn't exist, create it:
          *-- check if you don't have any _old_ dbf with the same 
          *-- name!!!
          cAlias = Alias()
          if .not. file("AUTOINC.DBF")
             select select()  && Next Work Area
             *-- structure has been changed so don't use any old 
             *-- version dbf! 
             *---------------------------------------------------------
             *-- WARNING: This is SQL code, and if you wish to
             *-- distribute an application with this class,
             *-- you must distribute the SQL portion of the BDE.
             *-- If you do not, this will fail. (You could modify
             *-- the code to use the XBase "Create From" ... but
             *-- it would make this much longer ...)
             *---------------------------------------------------------
             create table "AUTOINC.DBF" ( incfield Char (50),;
                                          incvalue Numeric(15,0))
             use autoinc excl && Use Exclusive To Create Index Tag
             
             *-- check if any SET FIELDS is active
             if set("FIELDS")="ON"
                *-- SET FIELDS so that VdB sees them
                set fields to incvalue, incfield
             endif && Set("FIELDS")="ON"
          
             *-- create index tag, so that we can use one record for 
             *-- each field that needs an incrementing value
             index on upper(incfield) tag incfield
             *-- close newly created dbf and return to the saved 
             *-- workarea
             use
             select (cAlias)
          endif && .Not. File("AUTOINC.DBF")
       
          *-- open autoinc.dbf, so we can increment it, then
          *-- close it as soon as possible:
          *-- create exact search expression so that we don't have to
          *-- bother with SET EXACT
          cSeek = left(upper(this.datalink)+space(50),50)
              
          select select()  && Next Work Area
          use autoinc order incfield    && Open The Table
          
          *-- check if any SET FIELDS is active
          if set("FIELDS")="ON"
             *-- SET FIELDS so that VdB sees them
             set fields to incvalue, incfield
          endif && Set("FIELDS")="ON"
          
          *-- wait loop for multiple users the same time
          do while .not. flock() && Try To Flock It, If Impossible
          enddo                  && Another User Was Faster
           
          *-- search record with this datalink
          if seek(cSeek)  && Ok Record Exist
             *-- increment value
             replace autoinc->incvalue with autoinc->incvalue+1
          else
             *-- it does not exist, so add a new record
             append blank
             *-- save datalink to this record and reset the counter
             replace autoinc->incvalue with 1,;
                     autoinc->incfield with this.datalink
          endif && Seek(Cseek)  &&Ok Record Exist
              
          do case
             case type("this.value")="N" &&It's a numeric field
                  *-- assign numeric value unchanged
                  this.value = autoinc->incvalue
             case type("this.value")="C" &&It's a char field
                  *-- construct the right picture clause for this field
                  *-- @L means leading zeroes
                  cPicture ="@L "+replicate("9",len(this.value))
                  *-- transform to chr type and lenght
                  this.value= transform(autoinc->incvalue, cPicture)
             otherwise
                *-- the user set the datalink to an illegal type field
                ?? chr(7)
                msgbox("AUTOINC.CC only handles N and C datatypes!",16)
          endcase
          use
          select (cAlias)
          
       RETURN .F.  && Disallow User Editing ...
       
       PROCEDURE AutoIncOnOpen
       *-- set the focus to every autoincrement control automatically 
       *-- each time a navigating occurs in the form
       
          if type("form.autoinc_already_installed")="U"
             *-- do this only once for the first autoincrement
             *-- control on the form.
             form.autoinc_already_installed=.T.
          
             *-- If the form does have no code specified to execute at 
             *-- the OnNavigate event.....
             if (empty(form.OnNavigate))
                form.OnNavigate=CLASS::CheckAuto
             else
                *-- otherwise execute both codes
                form.AutoOnNavigate = Form.OnNavigate
                *-- save reference to new property and
                Form.OnNavigate=;
                            {;form.AutoOnNavigate();class::checkauto()}
             
            endif && (Empty(Form.Onnavigate))
          endif && Type("form.autoinc_already_installed")="
       
       PROCEDURE CheckAuto
          *-- set focus to each autoincrement object on the form to 
          *-- check if it has a value
       
          *-- save current control to a variable
          oSControl = form.ActiveControl

          *-- loop through all controls and set focus to all the
          *-- autoincrement controls
          oControl=form.First
          do
             if "AUTOINC" $ oControl.name
                 oControl.IncrementIt()
             endif && "AUTOINC" $ Ocontrol.Name
             oControl = oControl.before
          until oControl.name = form.first.name

          *-- set focus to the control that had it before this loop
          oSControl.setfocus()

ENDCLASS       
*-- End of Class: AUTOINC

CLASS BigCursorEntryfield(FormObj,Name) OF ;
                                        ENTRYFIELD(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      When an instance of this entryfield gets focus, the cursor
   *      changes shape to one that is a bit more obvious than the
   *      default.
   *
   *-- Programmers: 
   *
   *      ???????
   *
   *-- History: 
   *
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"). 
   *----------------------------------------------------------------------

   Protect OnOpen,OnGotFocus

   PROCEDURE OnOpen
      if type("CREATECARET")# "FP"
         extern CVOID CreateCaret ( CHANDLE,CHANDLE,CINT,CINT ) USER
      endif
      if type("SHOWCARET")# "FP"
         extern CVOID ShowCaret ( CHANDLE ) USER
      endif
   
   PROCEDURE OnGotFocus
      createcaret(this.hwnd,0,5,24)  
      showcaret(this.hwnd)
      
ENDCLASS
*-- End of Class: Big_Cursor_EntryField

CLASS DateEpochEntry(FormObj,Name) OF ENTRYFIELD(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      A Custom Entry Field that, within a one century period, and 
   *      over the turn of the century, will derive the proper century 
   *      for two digit date entry.  It is based on the Epoch approach
   *      used in CA Clipper.  It uses the fact that VdB carries the 
   *      year as four digits even if SET CENTURY is off.
   *
   *      WARNING: Instantiaton of this class will set CENTURY off.
   *
   *-- Programmers: 
   *
   *      Thomas H. Arnold
   *
   *-- History: 
   *
   *      05/01/1996 -- Original (titled: DateField)
   *      06/01/1996 -- Updated by author
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"), and set the BaseDate property (see below) in the
   *      same place you set the datalink.
   *
   *-- Properties:
   *
   *     Basedate
   *       Must be set to a date such that all dates whose two digit 
   *       year is greater than the basedate, will result in a century 
   *       date the same as the basedate, and dates whose two digit 
   *       date is less that the basedate, will result in a century 
   *       date in the century following the basedate.  A suitable 
   *       value would be any four digit year less than the present 
   *       year, like 1960 (this is the default).
   *-------------------------------------------------------------------

   SET CENTURY OFF
   
   this.Valid = CLASS::DATEFIELD_VALID
   this.Value = {  /  /  }
   this.Top = 3
   this.Left = 8
   this.Picture = "99/99/99"
   this.Function = "D"
   this.Height = 1
   this.Width = 8.5
   this.BaseDate = 1960
   
   PROCEDURE DateField_Valid
  
      local cCentury
      this.date = dtoc( this.value )
      
      if year( this.value ) < this.basedate
         cCentury = ltrim( str( year( this.value ) + 100 ))
      else  
         cCentury = ltrim( str( year( this.value )))
      endif   
      
      if SET( "DATE" ) == "JAPAN" .OR. SET( "DATE" ) == "ANSI" ;
                                  .OR. SET( "DATE" ) == "YMD"
         this.date = stuff( this.date, 1, 4, cCentury )      
      else
         this.date = stuff( this.date, 7, 4, cCentury )      
      endif
      
      this.value = ctod( this.date )
   
   RETURN .T.
   
ENDCLASS
*-- End of Class: DateEpochEntry

CLASS KeyFieldEntry(FormObj,Name) OF ENTRYFIELD(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      A Custom Entry Field that, when datalinked properly will
   *      ensure that entries will be unique ...
   *
   *-- Programmers: 
   *
   *      Ken Mayer
   *
   *-- History: 
   *
   *      05/01/1996 -- Created/modified for Borland Developer's
   *                    Conference ...
   *      12/06/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"). See other properties listed below -- all of these
   *      are required and should be set in the same place ... You may
   *      also wish to change the SpeedTip and ValidErrMsg properties 
   *      to messages specific to your use of the class.
   *
   *-- Properties:
   *
   *      These properties _must_ be handled by placing them in either
   *      the form's OnOpen, overriden Open/ReadModal, or the entry-
   *      field's OnOpen events ...
   *
   *--    this.datalink = "mytable->myfield"
   *--    this.cAlias   = "myalias"
   *--    this.cTag     = "mytag"
   *--    this.cField   = "myfield"
   *
   *     Replace 'mytable', 'myfield', 'myalias', 'mytag' with
   *     the appropriate values. 'MyTable' and 'MyAlias' can (and 
   *     usually are) be the same. 'MyTag' is the index tag that you are
   *     using to check that the field is unique. This requires that a 
   *     .MDX tag exist. And 'MyField' is the name of the field.
   *-------------------------------------------------------------------

    this.SpeedTip      = "The Value entered here must be unique!"
    this.Width         = 16
    this.Value         = ""
    this.Valid         = CLASS::IsUnique
    this.ValidRequired = .t.
    this.ValidErrMsg   = "This value must be unique. Please check "+;
                         "your entry and try again."

    *-- Custom Properties -- the designer must assign values to these 
    *-- in the entryfield's OnOpen 
    this.cAlias = ""   && alias name
    this.cTag   = ""   && .MDX tag name
    this.cField = ""   && field name

    PROCEDURE IsUnique
       local lUnique, cOldAlias
       lUnique = .t.         && assume true

       *---------------------------------------------
       *-- use the custom properties defined above to
       *-- check the uniqueness of the value of the
       *-- entryfield.
       *---------------------------------------------
       *-- open the table a second time, so we do not
       *-- mess things up:
       cOldAlias = alias()      && save current work area
       *-- open table a second time, in new work area, with
       *-- alias of "CheckIt" with the NoUpdate flag just to
       *-- be safe.
       use (this.cAlias) again in select() alias CheckIt NoUpdate
       select CheckIt           && don't leave this out!
       set order to (this.cTag)
       cFieldName = this.cField
       *-- perform a SEEK() on the value entered by the user,
       *-- in the appropriate .MDX tag.
       if seek(this.Value)
          *-- if match is found, we have to deal with the fact,
          *-- specifically when _editing_ the record, that the
          *-- value may be found in the .MDX tag for
          *-- the current record (i.e., don't declare an error
          *-- for the record you're on)
          *-- This is done by checking the contents of the field
          *-- when we hit with the SEEK() function against the
          *-- contents of the entryfield. If they're identical,
          *-- we have to check the record numbers. If those are
          *-- not identical, we have a match, and must give
          *-- an error (setting lUnique to .F.).
          do while &cFieldName. = this.value .and. .not. eof()
             if bookmark() # bookmark(this.cAlias)
                lUnique = .f.  && it's not unique
                exit           && get out of loop
             endif
             skip
          enddo
       endif && seek(...)
       *-- close the second copy of the table
       select CheckIt
       use
       *-- back to the original ...
       select (cOldAlias)
       
    RETURN lUnique

ENDCLASS

CLASS MultipleEntry( fArg, cName) OF ENTRYFIELD(fArg, cName) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      This is a multiple entry entryfield, sort of a replacement 
   *      for the dBASE/DOS @M picture/function.
   *
   *-- Programmers: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      11/21/1995 -- Original
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"). 
   *-------------------------------------------------------------------

      PROTECT selectall,key,onopen
      this.SelectAll       = .f.
      this.choices         = {"blue","red","green"}
      this.separator       ="|"
      this.nchoice         = 1
      this.statusmessage   = "Press <space> to alternate between "+;
                             "multiple choices"

      PROCEDURE Key( nChar, nPosition )
         if nChar=32 .and. alen(this.choices)>1
            if this.nChoice < alen(this.choices)
               this.nChoice=this.nChoice+1
            else
               this.nChoice=1
            endif
            this.value=this.choices[this.nChoice]
                
            nChar=0
         else
            if nChar > 32 .and. nChar =< 255
               nChar = 0
            endif
         endif
      RETURN nChar

ENDCLASS
*-- End of Class: MULTIPLEENTRY

CLASS SeqValue(F,N) OF ENTRYFIELD(F,N) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      Place this on a form, and make sure you datalink it to a 
   *      field in one of your tables -- the field must be numeric or 
   *      character. It should handle any number of fields of forms.
   *      This custom entryfield will fill automatically with 
   *      sequential values when issuing Form.Saverecord() 
   *      in the form it is placed. Used for InVoice/Order N#
   *      etc. where the sequence of the numbers must be guaranteed.
   *
   *-- WARNINGS:
   *
   *   1) This (and AUTOINCREMENT above) uses the SQL command to create
   *      the table used -- if you need to deploy an application
   *      using this class, you need to deploy the SQL portion of
   *      the BDE, or your application will fail. You may, instead
   *      modify the code to use the standard dBASE "CREATE FROM"
   *      command, but this will take a bit more work ...
   *   2) Your form must use the FORM.beginAppend() and 
   *      form.SaveRecord() events for this to work. This class over-
   *      rides the form's SaveRecord event.
   *
   *-- Programmers: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set procedure to entry.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *      set the datalink as noted in the header of this file (under
   *      "NOTE"). You may also wish to modify the SpeedTip, but to do
   *      so, you will need to do this in the code below, as it is
   *      a "protected" property, and once you have an instance of
   *      the object, you can't change it ...
   *----------------------------------------------------------------------
      
      protect OnOpen, When, Speedtip
      this.speedtip  = "SeqValue Custom Control -- You Can't Edit Me!"
      this.when      = {;Return .F.}
      this.OnOpen    = CLASS::SeqValueOnopen
       
      PROCEDURE IncrementIt
         *-- This control MUST have a datalink
         if empty(this.datalink)
            ?? Chr(7)
            msgbox("There is no datalink for this control!","ERROR",16)
            RETURN .F.
         endif && Empty(This.Datalink)
       
         *-- Check if it already has a value
         if type("this.value")="N" .and. this.value > 0
            return .F.  && Don't do anything!
         endif && Type("this.value")="N" .And. This.Value 
         if type("this.value")="C" .and. .not. empty(this.value)
            return .F.  && Don't do anything!
         endif && Type("this.value")="C" .And. .Not. Empty
                
         *-- if table doesn't exist, create it:
         *-- check if you don't have any _old_ dbf with the same 
         *-- name!!!
         cAlias = Alias()
         if .not. file("SeqValue.DBF")
            select select()  && Next Work Area
            *-- structure has been changed so don't use any old version
            *-- dbf!
            create table "SeqValue.DBF" (incfield Char (50),;
                                         incvalue Numeric(15,0))
            use seqvalue excl &&Use Exclusive To Create Index Tag
            *-- check if any SET FIELDS is active
            if set("FIELDS")="ON"
               *-- SET FIELDS so that VdB sees them
               set fields to incvalue, incfield
            endif && Set("FIELDS")="ON"
          
            *-- create index tag, so that we can use one record for 
            *-- each field that needs an incrementing value
            index on upper(incfield) tag incfield
            *-- close newly created dbf and return to the saved 
            *-- workarea
            use
            select (cAlias)
         endif && .Not. File("SeqValue.DBF")
         
         *-- open SeqValue.dbf, so we can increment it, then
         *-- close it as soon as possible:
         *-- create exact search expression so that we don't have to
         *-- bother with SET EXACT
         cSeek = left(upper(this.datalink)+space(50),50)
       
         select select()  && Next Work Area
         use seqvalue order incfield    && Open The Table
         *-- check if any SET FIELDS is active
         if set("FIELDS")="ON"
            *-- SET FIELDS so that VdB sees them
            set fields to incvalue, incfield
         endif && Set("FIELDS")="ON"
         *-- wait loop for multiple users the same time
         do while .not. flock() && Try To Flock It, If Impossible
         enddo                  &&Another User Was Faster
         *-- We lock the _file_ instead of the record (RLOCK()) to 
         *-- avoid problems when 2 users would try to add a new record 
         *-- to SEQVALUE the same time for the same table field when 
         *-- used the first time.
       
         *-- search record with this datalink
         if seek(cSeek)  &&Ok Record Exist
            *-- increment value
            replace SeqValue->Incvalue with SeqValue->incvalue+1
         else
            *-- it does not exist, so add a new record
            append blank
            *-- save datalink to this record and reset the counter
            *-- Here's where the problem could happen if we used record 
            *-- lock instead on file lock, 2 users might add a record the 
            *-- same table-field
            replace seqValue->Incvalue with 1,;
                    seqvalue->Incfield with this.datalink
         endif && Seek(Cseek)  &&Ok Record Exists
         
         do case
            case type("this.value")="N" &&It's a numeric field
                 *-- assign numeric value unchanged
                 this.value = SeqValue->Incvalue
            case type("this.value")="C" &&It's a char field
                 *-- construct the right picture clause for this field
                 *-- @L means leading zeroes
                 cPicture ="@L "+replicate("9",len(this.value))
                 *-- transform to chr type and lenght
                 this.value= transform(SeqValue->Incvalue, cPicture)
            otherwise
               *-- the user set the datalink to an illegal type field
               ?? chr(7)
               msgbox("SeqValue.CC only handles N and C datatypes!")
          endcase
          use
          select (cAlias)
          *-- so we just updated the value, now save it with the rest 
          *-- of the changes that were done.
          form.saveRecord()
       
       RETURN .F.  
       
       PROCEDURE SeqValueOnOpen
          *-- set the focus to every SeqValuerement control 
          *-- automatically each time a navigating occurs in the form
                  
          if type("form.SeqValue_already_installed")="U"
             *-- do this only once for the first SeqValuerement
             *-- control on the form.
             form.SeqValue_already_installed=.T.
          
             *-- If the form does have no code specified to execute at 
             *-- the OnNavigate event.....
             if (empty(form.SaveRecord))
                form.SaveRecord=CLASS::CheckAuto
             else
                *-- otherwise execute both codes
                form.AutoSaveRecord = form.SaveRecord
                *-- save reference to new property and
                form.SaveRecord=;
                            {;form.AutoSaveRecord();CLASS::CheckAuto()}
             endif && (Empty(Form.Onnavigate))
          endif && Type("form.SeqValue_already_installed")="
       
       PROCEDURE Checkauto
          *-- set focus to each SeqValue object on the form to check
          *-- if it has a value
       
          *-- save current control to a variable
          oSControl = form.ActiveControl
          *-- loop through all controls and set focus to all the
          *-- SeqValue controls
          oControl = form.First
          do
            if "SEQVALUE" $ upper(oControl.name)
               oControl.IncrementIt()
            endif && "SeqValue" $ Ocontrol.Name
            oControl = oControl.Before
          until oControl.name = form.first.Name
          *set focus to the control that had it before this loop
          oScontrol.Setfocus()
ENDCLASS
*-- End of Class: SEQValue

*----------------------------------------------------------------------
*-- End of File: ENTRY.CC
*----------------------------------------------------------------------